/* 
 * Tagfu - An Entity Tagging System
 *   * libtagfu - A C implementation of TagFu
 * 
 * Copyright (c) 2006, Alexander Botero-Lowry
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * * Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * * Neither the name of the authors explicitly named above, the name of the
 *   Tagfu Team, nor the names of its contributors may be used to endorse or
 *   promote products derived from this software without specific prior written
 *   permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 */
#include "tagfu.h"

sqlite3 *tagfu_context_session_new(tagfu_context_t *context);
int tagfu_context_session_end(tagfu_context_t *context);



static int _tagfu_tags_get_name_cb(void *result, int num_col, char **res_val, char **res_key);
static int _tagfu_entities_get_path_cb(void *result, int num_col, char **res_val, char **res_key);

sqlite3 *tagfu_context_session_new(tagfu_context_t *context) {
	/*
	 * we open a sqlite context, set it on the struct, and reutrn 
     * depending on if the context was open. Make sure to also set a timeout
	 * for threading reasons.
	 */
	sqlite3 *sql;
	char database_path[20];

	sprintf(database_path, "/home/alex/.tagfu/tagfu.db");
	if (sqlite3_open (database_path, &sql)) {
		printf("open failed");
		return NULL;
	}
	sqlite3_busy_timeout (sql, 55000);
	return sql;
}

int tagfu_context_session_end(tagfu_context_t *context) {
	sqlite3_close(context->sql);
}

tagfu_context_t *tagfu_init() {
	tagfu_context_t *context;
	context = calloc(1, sizeof(tagfu_context_t));
	return context;
}

int tagfu_deinit(tagfu_context_t *context) {
	free(context);
}

int tagfu_entities_list(tagfu_context_t *context, int *id_list) {
	return;
}

int tagfu_entities_list_length(tagfu_context_t *context) {
	return;
}

int tagfu_entities_add(tagfu_context_t *context, char *path) {
	return;
}

int tagfu_entities_id_from_path(tagfu_context_t *context, char *path) {
	return;
}

char *tagfu_entities_get_path(tagfu_context_t *context, int entity_id) {
	char query[256];
	char *errmsg, *result;
	int ret;

	result = calloc(2048, sizeof(char));

	context->sql = tagfu_context_session_new(context);

	snprintf(query, sizeof(query), "SELECT url from Entity where id='%d';", 
	         entity_id);
	ret = sqlite3_exec(context->sql, query, _tagfu_entities_get_path_cb, 
	                   (void *) result, &errmsg);
	if( ret != SQLITE_OK ) {
		return NULL;
	}
	
	tagfu_context_session_end(context);
	return result;
}

static int _tagfu_entities_get_path_cb(void *result, int num_col, 
                                       char **res_val, char **res_key) {
	sprintf((char *) result, "%s", res_val[0]);
	return 0;
}

int tagfu_entities_delete(tagfu_context_t *context, int entity_id) {
	return;
}

int tagfu_entities_tags_list(tagfu_context_t *context, int entity_id, 
                             char *tag_list) {
	return;
}

int tagfu_entities_tags_list_length(tagfu_context_t *context, int entity_id) {
	return;
}

char *tagfu_entities_tags_property_get(tagfu_context_t *context, int entity_id, 
                                     int tag_id) {
	return;
}

int tagfu_entities_tags_property_set(tagfu_context_t *context, int entity_id,
                                     int tag_id, char *value) {
	return;
}
int tagfu_entities_tags_property_delete(tagfu_context_t *context, int entity_id,
                                        int tag_id) {
	return;
}

int tagfu_tags_list(tagfu_context_t *context, int *id_list) {
	return;
}

int tagfu_tags_list_length(tagfu_context_t *context) {
	return;
}

char *tagfu_tags_get_name(tagfu_context_t *context, int tag_id) {
	char query[256];
	char *errmsg, *result;
	int ret;

	result = calloc(256, sizeof(char));

	context->sql = tagfu_context_session_new(context);

	snprintf(query, sizeof(query), "SELECT name from Tag where id='%d';", 
	         tag_id);
	ret = sqlite3_exec(context->sql, query, _tagfu_tags_get_name_cb, 
	                   (void *) result, &errmsg);
	if( ret != SQLITE_OK ) {
		return NULL;
	}
	
	tagfu_context_session_end(context);
	return result;
}

static int _tagfu_tags_get_name_cb(void *result, int num_col, char **res_val, char **res_key) {
	sprintf((char *) result, "%s", res_val[0]);
	return 0;
}

int tagfu_tags_add(tagfu_context_t *context, char *name) {
	return;
}

int tagfu_tags_delete(tagfu_context_t *context, int tag_id) {
	return;
}

int tagfu_tags_id_from_name(tagfu_context_t *context, char *name) {
	return;
}
