/* 
 * Tagfu - An Entity Tagging System
 *   * libtagfu - A C implementation of TagFu
 * 
 * Copyright (c) 2006, Alexander Botero-Lowry
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * * Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * * Neither the name of the authors explicitly named above, the name of the
 *   Tagfu Team, nor the names of its contributors may be used to endorse or
 *   promote products derived from this software without specific prior written
 *   permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <sqlite3.h>

struct tagfu_context_St {
	sqlite3 *sql;
	int write;
};

typedef struct tagfu_context_St tagfu_context_t;

tagfu_context_t *tagfu_init();
int tagfu_deinit(tagfu_context_t *context);
int tagfu_entities_list(tagfu_context_t *context, int *id_list); /* boolean return */
int tagfu_entities_list_length(tagfu_context_t *context); /* return length */
int tagfu_entities_add(tagfu_context_t *context, char *path); /* boolean return */
int tagfu_entities_id_from_path(tagfu_context_t *context, char *path); /* return id */
char *tagfu_entities_get_path(tagfu_context_t *context, int entity_id); /* boolean return */
int tagfu_entities_delete(tagfu_context_t *context, int entity_id); /* boolean return */
int tagfu_entities_tags_list(tagfu_context_t *context, int entity_id, char *tag_list); /* boolean return */
int tagfu_entities_tags_list_length(tagfu_context_t *context, int entity_id); /* return length */
char *tagfu_entities_tags_property_get(tagfu_context_t *context, int entity_id, int tag_id); /* boolean return */
int tagfu_entities_tags_property_set(tagfu_context_t *context, int entity_id, int tag_id, char *value); /* boolean return */
int tagfu_entities_tags_property_delete(tagfu_context_t *context, int entity_id, int tag_id); /* boolean return */
int tagfu_tags_list(tagfu_context_t *context, int *id_list); /* boolean return */
int tagfu_tags_list_length(tagfu_context_t *context); /* return length */
char *tagfu_tags_get_name(tagfu_context_t *context, int tag_id); /* boolean return */
int tagfu_tags_add(tagfu_context_t *context, char *name); /* boolean return */
int tagfu_tags_delete(tagfu_context_t *context, int tag_id); /* boolean return */
int tagfu_tags_id_from_name(tagfu_context_t *context, char *name); /* return id */
